"""
Python implementation for Java application UI automation with Java Access Bridge.
"""
__author__ = "Gary Gao"
__email__ = "gaozhao89@qq.com"
__license__ = "GPLv2"
__url__ = "https://github.com/gaozhao1989/pyjab"
__version__ = "1.0.5"
